/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
* @brief ADXL375 I2C Driver.
* @defgroup 
*
*/

#include <stdbool.h>
#include "SEGGER_RTT.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"
#include "app_util_platform.h"
#include "app_error.h"
#include "boards.h"
#include "ADXL375_I2C.h"
#include <stdarg.h>

static const nrf_drv_twi_t * p_twi_master;

 ret_code_t ADXL375_I2C_register_write(uint8_t slave_addr,uint8_t * pdata, uint32_t bytes)
{
    ret_code_t ret_code;

    ret_code = nrf_drv_twi_tx(p_twi_master, slave_addr, pdata, bytes , false);
	
    return ret_code;
}

 ret_code_t ADXL375_I2C_register_read(uint8_t slave_addr, uint8_t reg_addr,  uint8_t * pdata, uint32_t bytes)
{   
    ret_code_t ret_code;
    
    ret_code = nrf_drv_twi_tx(p_twi_master,slave_addr, &reg_addr,1,false);
    
    if(ret_code != NRF_SUCCESS)
    {
        return ret_code;
    }
    
    ret_code = nrf_drv_twi_rx(p_twi_master,slave_addr, pdata, bytes);
    
    return ret_code;
}

 void ADXL375_I2C_offsets_set(uint8_t x_offset, uint8_t y_offset,uint8_t z_offset)
{
    ret_code_t ret_code;
    
    uint8_t tx_data[4] = {OFSX, x_offset,y_offset,z_offset};
    
    ret_code = ADXL375_I2C_register_write(ADXL375_ADDR, tx_data, sizeof(tx_data));
    
    APP_ERROR_CHECK(ret_code);
    
    uint8_t rx_data[3] ={0x00,0x00,0x00};
    
    ret_code = ADXL375_I2C_register_read(ADXL375_ADDR, OFSX, rx_data, sizeof(rx_data)); 
    
    APP_ERROR_CHECK(ret_code);
    
    
    if( rx_data[0] == x_offset &&
        rx_data[1] == y_offset &&
        rx_data[2] == z_offset  )
    {
        SEGGER_RTT_printf(0,"Setting Offsets Sucessful! \n");
    }
    else
    {
        SEGGER_RTT_printf(0,"Setting Offsets Failed! \n");
    }
}

 void ADXL375_I2C_data_rate_set(uint8_t rate)
{
    ret_code_t ret_code;
    
    uint8_t tx_data[2] = {BW_RATE,rate};
    
    ret_code = ADXL375_I2C_register_write(ADXL375_ADDR,tx_data, sizeof(tx_data));

    APP_ERROR_CHECK(ret_code);
    
    uint8_t rx_data = 0x00;

    ret_code = ADXL375_I2C_register_read(ADXL375_ADDR, BW_RATE, &rx_data,1);

    APP_ERROR_CHECK(ret_code);
    
    if(rx_data == rate)
    {
        SEGGER_RTT_printf(0,"Setting Data rate Sucessful! \n");
    }
    else
    {
        SEGGER_RTT_printf(0,"Setting Data rate Failed! \n");
    }
}

 ret_code_t ADXL375_I2C_init(const nrf_drv_twi_t * p_twi_instance)
{
    ret_code_t ret_code;
    
    p_twi_master = p_twi_instance;
    
    uint8_t tx_data[2] = {POWER_CTL, 0x08}; 
    
    // Put the ADXL375 in Measurement Mode by writing 0x08 to the POWER_CTL Register
    ret_code = ADXL375_I2C_register_write(ADXL375_ADDR, tx_data, sizeof(tx_data) );

    return ret_code;
}

void ADXL375_I2C_acc_data_read(int16_t * x_value, int16_t * y_value, int16_t * z_value)
{
    ret_code_t ret_code;
    
    uint8_t rx_buffer[6];
    
    ret_code = ADXL375_I2C_register_read(ADXL375_ADDR, DATAX0, rx_buffer, sizeof(rx_buffer)); 
    
    APP_ERROR_CHECK(ret_code);
    
    *x_value = (rx_buffer[1]<<8)|rx_buffer[0];
    *y_value = (rx_buffer[3]<<8)|rx_buffer[2];
    *z_value = (rx_buffer[5]<<8)|rx_buffer[4];
}

ret_code_t fetch_HR(uint8_t * hr){
		
    ret_code_t ret_code;
		
    uint8_t returned_over_I2C[3]; //Array to hold returned data

		uint8_t reg[2] = {0x47, 0x01}; // <0x47><N> - request N = 1 HR samples
		
    ret_code = nrf_drv_twi_tx(p_twi_master, HRMI_I2C_ADDR, reg, sizeof(reg), false);

		if(ret_code != NRF_SUCCESS)
    {
        return ret_code;
    }
    
		if(ret_code == NRF_SUCCESS)
    {
        SEGGER_RTT_printf(0,"HRMI tx success");
    }
		
    ret_code = nrf_drv_twi_rx(p_twi_master,HRMI_I2C_ADDR, returned_over_I2C, sizeof(returned_over_I2C));
    
		*hr = returned_over_I2C[2];
		
    return ret_code;
}

/*ret_code_t start_HR(uint8_t slave_addr, uint8_t returned_over_I2C, uint32_t bytes) 
{
    ret_code_t ret_code;
    
		uint8_t reg[2] = {0x47, 0x01}; // <0x47><N>
    		
    ret_code = nrf_drv_twi_tx(p_twi_master, HRMI_I2C_ADDR, reg, sizeof(reg), false);
    SEGGER_RTT_printf(0,"no success");
    if(ret_code != NRF_SUCCESS)
    {
				return ret_code;
    }
    
    ret_code = nrf_drv_twi_rx(p_twi_master, HRMI_I2C_ADDR, &returned_over_I2C, sizeof(returned_over_I2C));
    
    return ret_code;
}
*/

/* //Set HR algorithm mode - I2C command <0x53><N> where <N> is 3 bit address and bit position zero is '1' to set mode to average mode

ret_code_t HRMI_set_mode(const nrf_drv_twi_t * p_twi_instance) {
    ret_code_t ret_code;
		p_twi_master = p_twi_instance;
    
    uint8_t reg[2] = {0x53, 0x01}; // <0x53><N> 
    
    // Put the ADXL375 in Measurement Mode by writing 0x08 to the POWER_CTL Register
    ret_code = ADXL375_I2C_register_write(HRMI_I2C_ADDR, reg, sizeof(reg) ); 

    return ret_code;
}*/